/*
 * Redis on NoSQL-tietovarasto joka soveltuu erityisen hyvin sellaisen datan säilytykseen jota
 * ei tarvitse säilyttää pitkäaikaisesti ja joka muuttuu nopeasti. Redis on erittäin nopea.
 *
 * Redis asennetaan Windowsille esim. msi-paketista jonka saa osoitteesta:
 * https://github.com/MSOpenTech/redis/releases
 *
 * Redis serveri käynnistetään komentokehotteella, esim: C:/Program Files/Redis>redis-server.exe
 * ja client C:/Program Files/Redis>redis-cli.exe jos Redistä tarvitsee hallinnoida komentokehotteesta
 * Rediksen hallinnointi on helpompaa GUI:lla kuten http://redisdesktop.com/
 *
 * Redis säilyttää dataa muistissa ja pitää kiintolevyllä logia datan muutoksista. Tarvittaessa
 * data voidaan palauttaa kiintolevyltä.
 *
 * Rediksen yleisimmät tietorakenteet:
 *
 * String on merkkijono jolla voi olla arvo. Esim. "color": "red"
 * HashTable on taulukko joka sisältää avain-arvo -pareja eli assosiatiivinen taulukko.
 * List on merkkijonolista joka on siinä järjestyksessä missä data on lisätty
 * Set on indeksitön eli järjestämätön merkkijonolista jossa ei voi olla kahta samaa alkiota.
 *
 * Nodella luodaan redis-client ja sitten clientin komennoilla (kuten set, hSet, hGet yms.)
 * manipuloidaan kannassa olevia tietorakenteita.
 *
 *  Redis komennot: http://redis.io/commands
 *
 */

const redis = require('redis');

(async () => {
  const client = redis.createClient(6379, '127.0.0.1'); //client portissa 6379 localhostina

  client.on('error', (err) => console.log('Redis Client Error', err));

  await client.connect();

  // set-komennolla laitetaan string-tyyppinen avain-arvopari kantaan
  await client.set('nimi', 'Tommi');
  const value1 = await client.get('nimi'); // haetaan arvo
  console.log('Nimi on ' + value1);

  //hSet -komennolla hash-table kantaan
  await client.hSet('safka', {
    leikkeleet: 'savustettu saunapalvikinkku',
    vihannekset: 'rasvaa tihkuva uunimakkara',
  });

  //hGet hakee arvon hash-tablesta
  const value2 = await client.hGet('safka', 'vihannekset');
  console.log('Tänään syödään ' + value2);

  //hKeys hakee avaimet hashtablesta
  const keys = await client.hKeys('safka');
  console.log('Varastossa ovat seuraavat safkalajit: ');

  keys.forEach((key) => {
    console.log(' ' + key);
  });

  // lPush laittaa listaan tavaraa
  await client.lPush('duunit', 'Maalaa seinä punaiseksi!');
  await client.lPush('duunit', 'Maalaa seinä siniseksi!');

  // lRange hakee kaikki listan alkiot: 0-eka -1-vika
  const items = await client.lRange('duunit', 0, -1);
  // tulostetaan kaikki alkiot
  items.forEach((item) => {
    console.log(' ' + item);
  });

  // sAdd lisää alkion settiin
  await client.sAdd('setti', 'Hello!');
  await client.sAdd('setti', 'Haloo!');

  const members = await client.sMembers('setti');
  console.log(members);
})();
